#include "hatpPlan.hh"

using namespace std;

hatpPlan::hatpPlan(string const& plan){
	boost::smatch what; 
	istringstream idTemp;
	this->validPlan = true;
	if(boost::regex_match(plan, what, regexPlan)) {
		idTemp.str(what[1]); 
		idTemp >> this->planID;
		this->planReport = what[2];
		this->constructNodeList(what[3]);
		this->constructLinkList(what[47], STREAM_TYPE);
		string rootTTemp = what[57];
		this->constructLinkList(what[63], TREE_TYPE);
		string rootSTemp = what[41];
		this->streams = new hatpStreams(tokenizeInt(removeFormatting(rootSTemp), ","), this->getActionNodes());
		vector<unsigned int> rootIDTemp = tokenizeInt(removeFormatting(rootTTemp), ",");
		this->tree = new hatpTree(rootIDTemp[0], this->getNodes());
	}
	else if(boost::regex_search(plan, what, regexSNoPlan)) {
		idTemp.str(what[1]); 
		idTemp >> this->planID;
		this->planReport = REPORT_NOPLAN;	
		this->streams = NULL;
		this->tree = NULL;
	}
	else if(boost::regex_search(plan, what, regexSAchieved)) {
		idTemp.str(what[1]); 
		idTemp >> this->planID;
		this->planReport = REPORT_ACHIEVED;	
		this->streams = NULL;
		this->tree = NULL;
	}
	else validPlan = false;
}

bool hatpPlan::isPlanValid() const {
	return this->validPlan;
}

bool hatpPlan::isReportOK() const {
	return (this->planReport == REPORT_OK);
}

bool hatpPlan::isReportNoPlan() const {
	return (this->planReport == REPORT_NOPLAN);
}

bool hatpPlan::isReportAlreadyAchieved() const {
	return (this->planReport == REPORT_ACHIEVED);
}

void hatpPlan::constructNodeList(string const& content) {
	const int act_subs[] = {1, 2, 3, 4, 10, 11, 12};
	unsigned int ncount = 1, nodeId;
	double stime, etime;
	string nid, nty, nna, nag, nst, net, npa;
	istringstream idTemp, stTemp, etTemp;
	boost::sregex_token_iterator regI(content.begin(), content.end(), regexNode, act_subs), regJ;
	while(regI != regJ) {
		if(ncount % 7 == 1) nid = *regI++;
		if(ncount % 7 == 2) nty = *regI++;
		if(ncount % 7 == 3) nna = *regI++;
		if(ncount % 7 == 4) nag = *regI++;
		if(ncount % 7 == 5) nst = *regI++;
		if(ncount % 7 == 6) net = *regI++;
		if(ncount % 7 == 0) { 
			npa = *regI++;
			idTemp.clear(); idTemp.str(nid); idTemp >> nodeId;
			stTemp.clear(); stTemp.str(nst); stTemp >> stime;
			etTemp.clear(); etTemp.str(net); etTemp >> etime;
			removeQuotes(removeFormatting(nag));
			removeQuotes(removeFormatting(npa));
			hatpNode* newNode = new hatpNode(nodeId, nty, nna, tokenizeStr(nag, ","), stime, etime, tokenizeStr(npa, ","));
			this->nodes[nodeId] = newNode;
		}
		ncount++;
	}
}

void hatpPlan::constructLinkList(string const& content, string const& type) {
	const int act_subs[] = {1, 2, 3};
	unsigned int ncount = 1, nodeF, nodeT;
	string nty, nfr, nto;
	istringstream frTemp, toTemp;
	boost::sregex_token_iterator regI(content.begin(), content.end(), regexLink, act_subs), regJ;
	while(regI != regJ) {
		if(ncount % 3 == 1) nty = *regI++;
		if(ncount % 3 == 2) nfr = *regI++;
		if(ncount % 3 == 0) { 
			nto = *regI++;
			removeFormatting(nty);
			frTemp.clear(); frTemp.str(nfr); frTemp >> nodeF;
			toTemp.clear(); toTemp.str(nto); toTemp >> nodeT;
			hatpNode* nodeFrom = this->nodes.find(nodeF)->second;
			hatpNode* nodeTo = this->nodes.find(nodeT)->second;
			if(nty == LINK_CAUSAL) {
				if(type == TREE_TYPE) {
					nodeFrom->addTreeSuccessor(nodeT);
					nodeTo->addTreePredecessor(nodeF);
				}
				else {
					nodeFrom->addStreamSuccessor(nodeT);
					nodeTo->addStreamPredecessor(nodeF);
				}
			}
			else {
				nodeFrom->addSubNode(nodeT);
				nodeTo->setRootNode(nodeF);
			}
		}
		ncount++;
	}
}

unsigned int hatpPlan::getID() const {
	return this->planID;
}

string hatpPlan::getReport() const {
	return this->planReport;
}

vector<hatpNode*> hatpPlan::getNodes() const {
	vector<hatpNode*> result;
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end(); it++) {
		result.push_back((*it).second);
	}
	return result;
}

vector<hatpNode*> hatpPlan::getActionNodes() const {
	vector<hatpNode*> result;
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end(); it++) {
		if((*it).second->isAction()) result.push_back((*it).second);
	}
	return result;
}

hatpNode* hatpPlan::getNode(unsigned int n) const {
	return this->nodes.find(n)->second;
}

hatpStreams* hatpPlan::getStreams() const {
	return this->streams;
}

hatpTree* hatpPlan::getTree() const {
	return this->tree;
}

vector<string> hatpPlan::getAgentsList() const {
	vector<string> result;
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end() ; it++) {
		vector<string> agents = (it->second)->getAgents();
		for(unsigned int i = 0 ; i < agents.size() ; i++) {
			bool agentAlreadyPresent = false;
			for(unsigned int j = 0 ; j < result.size() ; j++) {
				if(result[j] == agents[i]) agentAlreadyPresent = true;
			}
			if(!agentAlreadyPresent) result.push_back(agents[i]);
		}
	}
	return result;
}

string hatpPlan::toString() const {
	ostringstream result;
	result << "===== BEGIN PLAN =====\n";
	result << "PLAN ID: " << this->planID << "\n";
	result << "PLAN REPORT: " << this->planReport << "\n";
	result << "PLAN NODES: \n";
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end(); it++) {
		result << (*it).second->toString() << "\n";
	}
	result << "STREAMS ROOTS: ["; 
	for(unsigned int i = 0 ; i < this->streams->getRootIDs().size() ; i++ ) result << this->streams->getRootIDs()[i] << " ";
	result << "]\n";
	result << "TREE ROOT: [" << this->tree->getRootID() << "]\n";
	result << "===== END PLAN =======\n";
	return result.str();
}

hatpPlan::~hatpPlan() {}


