#include "hatpStreams.hh"

using namespace std;

hatpStreams::hatpStreams(vector<unsigned int> r, vector<hatpNode*> n) {
	for(unsigned int i = 0 ; i < n.size() ; i++) {
		this->nodes[n[i]->getID()] = n[i];
	}
	this->roots = r;
	this->maxRank = -1;
	for(map<unsigned int, hatpNode*>::iterator it = this->nodes.begin() ; it != this->nodes.end() ; it++) {
		this->computeNodeRank((*it).second);
	}
}

hatpStreams::~hatpStreams() {}

void hatpStreams::computeNodeRank(hatpNode* cnode) {
	if(this->getRankOfNode(cnode->getID()) > -1) return;
	int cRank = 0;
	//TODO unsigned int cRank = 0;
	vector<unsigned int> pred = cnode->getStreamPredecessors();
	for(unsigned int i = 0 ; i < pred.size() ; i++) {
		hatpNode* ctemp = this->nodes.find(pred[i])->second;
		this->computeNodeRank(ctemp);
		if(this->getRankOfNode(ctemp->getID()) >= cRank) cRank = this->getRankOfNode(ctemp->getID()) + 1;
	}
	this->ranks.insert(make_pair(cnode->getID(), cRank));
	if(this->maxRank < (cRank)) this->maxRank = cRank;
}

vector<unsigned int> hatpStreams::getRootIDs() const {
	return this->roots;
}

hatpNode* hatpStreams::getNode(unsigned int id) const {
	std::map<unsigned int, hatpNode*>::const_iterator it(this->nodes.find(id));
	if(it==this->nodes.end())
		return NULL;
	return it->second;
}

vector<hatpNode*> hatpStreams::getRootNodes() const {
	vector<hatpNode*> result;
	for(unsigned int i = 0 ; i < this->roots.size() ; i++) {
		hatpNode* cnode = this->nodes.find(i)->second;
		result.push_back(cnode);
	}
	return result;
}

vector<hatpNode*> hatpStreams::getNodes() const {
	vector<hatpNode*> result;
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end(); it++) {
		result.push_back((*it).second);
	}
	return result;
}

unsigned int hatpStreams::getMaxRank() const {
	return this->maxRank;
}

int hatpStreams::getRankOfNode(unsigned int n) const {
//TODO unsigned int hatpStreams::getRankOfNode(unsigned int n) const {
	if(this->ranks.find(n)!=this->ranks.end()){
		return this->ranks.at(n);
	}
	return -1;
	//TODO return todo;
}

vector<hatpNode*> hatpStreams::getNodesOfRank(int r) const {
//TODO vector<hatpNode*> hatpStreams::getNodesOfRank(unsigned int r) const {
	vector<hatpNode*> result;
	for(std::map<unsigned int, int>::const_iterator it=ranks.begin(); it!=ranks.end(); ++it){
	//TODO for(std::map<unsigned int, unsigned int>::const_iterator it=ranks.begin(); it!=ranks.end(); ++it){
		if(it->second==r){
			if(this->nodes.find(it->first)!=this->nodes.end()) result.push_back(this->nodes.find(it->first)->second);
		}
	}
	return result;	
}

vector<hatpNode*> hatpStreams::getNodesForAgent(std::string ag) const {
	vector<hatpNode*> result;
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end() ; it++) {
		vector<string> agents = (it->second)->getAgents();
		for(unsigned int i = 0 ; i < agents.size() ; i++) {
			if(ag == agents[i]) result.push_back(it->second);
		}
	}
	return result;
}

vector<string> hatpStreams::getAgentsList() const {
	vector<string> result;
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end() ; it++) {
		vector<string> agents = (it->second)->getAgents();
		for(unsigned int i = 0 ; i < agents.size() ; i++) {
			bool agentAlreadyPresent = false;
			for(unsigned int j = 0 ; j < result.size() ; j++) {
				if(result[j] == agents[i]) agentAlreadyPresent = true;
			}
			if(!agentAlreadyPresent) result.push_back(agents[i]);
		}
	}
	return result;
}

vector<pair<unsigned int, unsigned int> > hatpStreams::getStreamsCausalLinks() const {
	vector<pair<unsigned int, unsigned int> > result;
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end() ; it++) {
		unsigned int currentNode = it->first;
		for(unsigned int i = 0 ; i < it->second->getStreamSuccessors().size() ; i++) {
			pair<unsigned int, unsigned int> pairRes = make_pair(currentNode, it->second->getStreamSuccessors()[i]);
			result.push_back(pairRes);
		}
	}
	return result;
}

string hatpStreams::toString() const {
	ostringstream result;
	result << "===== BEGIN STREAMS =====\n";
	result << "ROOTS ID: [";
	for(unsigned int i = 0 ; i < this->roots.size() ; i++) result << this->roots[i] << " ";
	result << "]\n";
	vector<string> agents = this->getAgentsList();
	for(unsigned int i = 0 ; i < agents.size() ; i++) {
		result << "NODES FOR AGENT " << agents[i] << ":";
		vector<hatpNode*> nodesAgent = this->getNodesForAgent(agents[i]);
		for(unsigned int j = 0 ; j < nodesAgent.size() ; j++) {
			result << " " << nodesAgent[j]->getID();
		}
		result << "\n";
	}
	result << "===== END STREAMS =======\n";
	return result.str();
}




