#include "hatpTree.hh"

using namespace std;

hatpTree::hatpTree(unsigned int r, vector<hatpNode*> n) {
	for(unsigned int i = 0 ; i < n.size() ; i++) {
		this->nodes[n[i]->getID()] = n[i];
	}
	this->root = r;
}

hatpTree::~hatpTree() {}

unsigned int hatpTree::getRootID() const {
	return this->root;
}

hatpNode* hatpTree::getRootNode() const {
	return this->nodes.find(this->root)->second;
}

vector<hatpNode*> hatpTree::getNodes() const {
	vector<hatpNode*> result;
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end(); it++) {
		result.push_back((*it).second);
	}
	return result;
}

// only root = 0 ; else >1 (1 by level) ;
unsigned int hatpTree::getTreeDepth() const{
	unsigned int result = 0, maxsd = 0;
	vector<unsigned int> subn = this->getRootNode()->getSubNodes();
	for(unsigned int i = 0 ; i < subn.size(); i++) {
		result = 1;
		hatpTree* subTree = this->getSubTree(this->nodes.find(subn[i])->second);
		unsigned int subd = subTree->getTreeDepth();
		if(subd > maxsd) maxsd = subd;
	}
	return result + maxsd;
}

hatpTree* hatpTree::getSubTree(hatpNode* r) const{
	vector<hatpNode*> nodesv;
	nodesv.push_back(r);
	vector<unsigned int> subnodes = r->getSubNodes();
	for(unsigned int i = 0 ; i < subnodes.size() ; i++) {
		vector<hatpNode*> tempv;
		tempv = (this->getSubTree(this->nodes.find(subnodes[i])->second))->getNodes();
		for(unsigned int j = 0 ; j < tempv.size() ; j++) {
			nodesv.push_back(tempv[j]);
		}
	}
	hatpTree* treeRes = new hatpTree(r->getID(), nodesv);
	return treeRes;
}

vector<hatpNode*> hatpTree::getNodesAtLevel(unsigned int level) const{
	vector<hatpNode*> result;
	if(level == 0) result.push_back(this->nodes.find(this->root)->second);
	else {
		vector<unsigned int> temp = this->getRootNode()->getSubNodes();
		for(unsigned int i = 0 ; i < temp.size() ; i++) {
			vector<hatpNode*> subRes = this->getSubTree(this->nodes.find(temp[i])->second)->getNodesAtLevel(level - 1);
			for(unsigned int j = 0 ; j < subRes.size() ; j++) {
				result.push_back(subRes[j]);
			}
		}
	}
	return result;
}


hatpTaskList* hatpTree::getRootAsTaskList() const {
	vector<unsigned int> rootv;
	rootv.push_back(this->root);
	return new hatpTaskList(this->nodes, rootv);
}

hatpTaskList* hatpTree::getSubNodesAsTaskList() const {
	return new hatpTaskList(this->nodes, this->getRootNode()->getSubNodes());
}

vector<hatpTaskList*> hatpTree::getTreeAsTaskLists() const {
	vector<hatpTaskList*> result = this->getTreeAsTaskLists(true);
	result.insert(result.begin(), this->getRootAsTaskList());
	return result;
}

vector<hatpTaskList*> hatpTree::getTreeAsTaskLists(bool first) const {
	vector<hatpTaskList*> result;
	hatpTaskList* subTL = this->getSubNodesAsTaskList();
	if(subTL->size() > 0) {
		result.push_back(subTL);
		map<unsigned int, hatpNode*> subn = subTL->getNodes();
		for(map<unsigned int, hatpNode*>::const_iterator it = subn.begin() ; it != subn.end() ; it++) {
			hatpTree* subtree = this->getSubTree((*it).second);
			vector<hatpTaskList*> subRes = subtree->getTreeAsTaskLists(false);
			for(unsigned int j = 0 ; j < subRes.size() ; j++) {
				if(subRes[j] > 0)
				result.push_back(subRes[j]);
			}
		}
	}
	return result;
}

vector<hatpTaskList*> hatpTree::getTaskListsAtLevel(unsigned int level) const{
	vector<hatpTaskList*> result;
	if(level == 0) {
		result.push_back(this->getRootAsTaskList());
		return result;
	}
	if(level == 1) {
		hatpTaskList* subTL = this->getSubNodesAsTaskList();
		if(subTL->size() > 0) result.push_back(subTL);
		return result;	
	}
	vector<unsigned int> temp = this->getRootNode()->getSubNodes();
	for(unsigned int i = 0 ; i < temp.size() ; i++) {
		vector<hatpTaskList*> subRes = this->getSubTree(this->nodes.find(temp[i])->second)->getTaskListsAtLevel(level - 1);
		for(unsigned int j = 0 ; j < subRes.size() ; j++) {
			result.push_back(subRes[j]);
		}
	}
	return result;
}

vector<pair<unsigned int, unsigned int> > hatpTree::getTreeCausalLinks() const {
	vector<pair<unsigned int, unsigned int> > result;
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end() ; it++) {
		unsigned int currentNode = it->first;
		for(unsigned int i = 0 ; i < it->second->getTreeSuccessors().size() ; i++) {
			pair<unsigned int, unsigned int> pairRes = make_pair(currentNode, it->second->getTreeSuccessors()[i]);
			result.push_back(pairRes);
		}
	}
	return result;
}

vector<pair<unsigned int, unsigned int> > hatpTree::getTreeHierarchicalLinks() const {
	vector<pair<unsigned int, unsigned int> > result;
	for(map<unsigned int, hatpNode*>::const_iterator it = this->nodes.begin() ; it != this->nodes.end() ; it++) {
		unsigned int currentNode = it->first;
		for(unsigned int i = 0 ; i < it->second->getSubNodes().size() ; i++) {
			pair<unsigned int, unsigned int> pairRes = make_pair(currentNode, it->second->getSubNodes()[i]);
			result.push_back(pairRes);
		}
	}
	return result;
}

string hatpTree::toString() const {
	ostringstream result;
	result << "===== BEGIN TREE =====\n";
	result << "ROOT ID: " << this->root << "\n";
	for(unsigned int i = 1 ; i <= this->getTreeDepth() ; i++) {
		result << "NODES ID AT LEVEL " << i << ":";
		std::vector<hatpNode*> subLnodes = this->getNodesAtLevel(i);
		for(unsigned int j = 0 ; j < subLnodes.size() ; j++) {
			result << " " << subLnodes[j]->getID();
		}
		result << "\n";
	}
	result << "===== END TREE =======\n";
	return result.str();
}




